@@ -137,12 +137,40 @@ static const char menu_def[] =
 "</popup>"
 "</ui>";
 
-static gboolean open_file( char* dir, GList* files, gpointer unused )
+static gboolean open_file( char* dir, GList* files, PtkFileBrowser* file_browser )
 {
     if( files )
     {
-        ptk_open_files_with_app( dir, g_list_copy( files ),
-                                 NULL, NULL, FALSE, TRUE );  //MOD
+        ptk_open_files_with_app( dir, files, NULL, NULL, FALSE, TRUE );
+
+        //sfm open selected dirs
+        if ( file_browser )
+        {
+            GList * l;
+            gchar* full_path;
+            VFSFileInfo* file;
+            
+            for ( l = files; l; l = l->next )
+            {
+                file = ( VFSFileInfo* ) l->data;
+                if ( G_UNLIKELY( ! file ) )
+                    continue;
+
+                full_path = g_build_filename( dir,
+                                              vfs_file_info_get_name( file ),
+                                              NULL );
+                if ( G_LIKELY( full_path ) )
+                {
+                    if ( g_file_test( full_path, G_FILE_TEST_IS_DIR ) )
+                    {
+                        ptk_file_browser_emit_open( file_browser,
+                                                            full_path, PTK_OPEN_NEW_TAB );
+                    }
+                    g_free( full_path );
+                }
+            }
+        }
+        vfs_file_info_list_free( files );  //sfm moved free list to here
         return TRUE;
     }
     return FALSE;
@@ -162,15 +190,21 @@ static void on_open_files( GtkAction* action, FindFile* data )
     GHashTable* hash;
     GtkWidget* w;
     VFSFileInfo* fi;
-
-    gboolean open_files = (0 == strcmp( gtk_action_get_name(action), "OpenAction") );
+    gboolean open_files_has_dir = FALSE;  //sfm
+    PtkFileBrowser* file_browser = NULL;  //sfm
+    gboolean open_files = TRUE;
+    
+    if ( action )
+        open_files = (0 == strcmp( gtk_action_get_name(action), "OpenAction") );
 
     sel = gtk_tree_view_get_selection( GTK_TREE_VIEW( data->result_view ) );
     rows = gtk_tree_selection_get_selected_rows( sel, &model );
     if( ! rows )
         return;
 
-    hash = g_hash_table_new_full( g_str_hash, g_str_equal, (GDestroyNotify)g_free, open_files ? (GDestroyNotify)vfs_file_info_list_free : NULL );
+    //sfm this frees list when new value inserted - caused segfault
+    //hash = g_hash_table_new_full( g_str_hash, g_str_equal, (GDestroyNotify)g_free, open_files ? (GDestroyNotify)vfs_file_info_list_free : NULL );
+    hash = g_hash_table_new_full( g_str_hash, g_str_equal, (GDestroyNotify)g_free, NULL );
 
     for( row = rows; row; row = row->next )
     {
@@ -189,7 +223,9 @@ static void on_open_files( GtkAction* action, FindFile* data )
                 GList *l;
                 l = g_hash_table_lookup( hash, dir );
                 l = g_list_prepend( l, vfs_file_info_ref(fi) );
-                g_hash_table_insert( hash, dir, l );
+                g_hash_table_insert( hash, dir, l );  //sfm caused segfault with destroy function
+                if ( vfs_file_info_is_dir( fi ) )  //sfm
+                    open_files_has_dir = TRUE;
             }
             else
             {
@@ -204,11 +240,23 @@ static void on_open_files( GtkAction* action, FindFile* data )
 
     if( open_files )
     {
-        g_hash_table_foreach_steal( hash, (GHRFunc)open_file, NULL );
+        if ( open_files_has_dir )
+        {
+            w = GTK_WIDGET( fm_main_window_get_on_current_desktop() );
+            if( ! w )
+            {
+                w = fm_main_window_new();
+                gtk_window_set_default_size( GTK_WINDOW( w ), app_settings.width, app_settings.height );
+            }
+            gtk_window_present( (GtkWindow*)w );
+            file_browser = (PtkFileBrowser*)fm_main_window_get_current_file_browser(
+                                                                (FMMainWindow*)w );
+        }
+        g_hash_table_foreach_steal( hash, (GHRFunc)open_file, file_browser );
     }
     else
     {
-        w = GTK_WIDGET( fm_main_window_get_last_active() );
+        w = GTK_WIDGET( fm_main_window_get_on_current_desktop() );
         if( ! w )
         {
             w = fm_main_window_new();
@@ -260,6 +308,8 @@ static char** compose_command( FindFile* data )
 
     arg = g_strdup( "find" );
     g_array_append_val( argv, arg );
+    arg = g_strdup("-H");
+    g_array_append_val( argv, arg );
 
     if( gtk_tree_model_get_iter_first( GTK_TREE_MODEL( data->places_list ), &it ) )
     {
@@ -437,7 +487,7 @@ static char** compose_command( FindFile* data )
         arg = g_strdup("grep");
         g_array_append_val( argv, arg );
 
-        if( gtk_toggle_button_get_active((GtkToggleButton*)data->fc_case_sensitive) )
+        if( !gtk_toggle_button_get_active((GtkToggleButton*)data->fc_case_sensitive) )
         {
             arg = g_strdup("-i");
             g_array_append_val( argv, arg );
@@ -485,7 +535,7 @@ static void finish_search( FindFile* data )
         g_object_unref( data->task );
         data->task = NULL;
     }
-    gdk_window_set_cursor( data->search_result->window, NULL );
+    gdk_window_set_cursor( gtk_widget_get_window( data->search_result ), NULL );
     gtk_widget_hide( data->stop_btn );
     gtk_widget_show( data->again_btn );
 }
@@ -608,9 +658,11 @@ static void on_start_search( GtkWidget* btn, FindFile* data )
     GError* err = NULL;
     int stdo, stde;
     char* cmd_line;
+    GtkAllocation allocation;
 
-    int width = GTK_WIDGET( data->win ) ->allocation.width;
-    int height = GTK_WIDGET( data->win ) ->allocation.height;
+    gtk_widget_get_allocation ( GTK_WIDGET( data->win ), &allocation );
+    int width =  allocation.width;
+    int height = allocation.height;
     if ( width && height )
     {
         char* str = g_strdup_printf( "%d", width );
@@ -643,7 +695,7 @@ static void on_start_search( GtkWidget* btn, FindFile* data )
         vfs_async_task_execute( data->task );
 
         busy_cursor = gdk_cursor_new( GDK_WATCH );
-        gdk_window_set_cursor( data->search_result->window, busy_cursor );
+        gdk_window_set_cursor( gtk_widget_get_window (data->search_result), busy_cursor );
         gdk_cursor_unref( busy_cursor );
     }
     else
@@ -658,14 +710,20 @@ static void on_stop_search( GtkWidget* btn, FindFile* data )
 {
     if( data->task && ! vfs_async_task_is_finished( data->task ) )
     {
+        // see note in vfs-async-task.c: vfs_async_task_real_cancel()
+        GDK_THREADS_LEAVE(); 
         vfs_async_task_cancel( data->task );
+        GDK_THREADS_ENTER();
     }
 }
 
 static void on_search_again( GtkWidget* btn, FindFile* data )
 {
-    int width = GTK_WIDGET( data->win ) ->allocation.width;
-    int height = GTK_WIDGET( data->win ) ->allocation.height;
+    GtkAllocation allocation;
+
+    gtk_widget_get_allocation ( GTK_WIDGET( data->win ), &allocation );
+    int width =  allocation.width;
+    int height = allocation.height;
     if ( width && height )
     {
         char* str = g_strdup_printf( "%d", width );
@@ -691,11 +749,14 @@ static void on_search_again( GtkWidget* btn, FindFile* data )
 
 static void menu_pos( GtkMenu* menu, int* x, int* y, gboolean *push_in, GtkWidget* btn )
 {
+    GtkAllocation allocation;
+
     /* FIXME: I'm not sure if this work well in different WMs */
-    gdk_window_get_position( btn->window, x, y);
+    gdk_window_get_position( gtk_widget_get_window( btn ), x, y);
 /*    gdk_window_get_root_origin( btn->window, x, y); */
-    *x += btn->allocation.x;
-    *y += btn->allocation.y + btn->allocation.height;
+    gtk_widget_get_allocation ( GTK_WIDGET( btn ), &allocation );
+    *x += allocation.x;
+    *y += allocation.y + allocation.height;
     *push_in = FALSE;
 }
 
@@ -737,7 +798,7 @@ static void on_add_search_desktop(GtkWidget* menu, FindFile* data)
 
 static void on_add_search_volumes(GtkWidget* menu, FindFile* data)
 {
-    char* path;
+    const char* path;
     const GList* vols = vfs_volume_get_all_volumes(), *l;
     for( l = vols; l; l = l->next )
     {
@@ -794,7 +855,8 @@ static void on_add_search_folder( GtkWidget* btn, FindFile* data )
     /* FIXME: Add all bookmarks */
 
     gtk_widget_show_all( menu );
-    gtk_menu_popup( GTK_MENU( menu ), NULL, NULL, menu_pos, btn, 0, gtk_get_current_event_time() );
+    gtk_menu_popup( GTK_MENU( menu ), NULL, NULL, (GtkMenuPositionFunc)menu_pos,
+                                        btn, 0, gtk_get_current_event_time() );
 }
 
 static void on_remove_search_folder( GtkWidget* btn, FindFile* data )
@@ -880,6 +942,21 @@ static gboolean on_view_button_press( GtkTreeView* view, GdkEventButton* evt, Fi
     {
         if( evt->button == 3 ) /* right single click */
         {
+            //sfm if current item not selected, unselect all and select it
+            GtkTreePath *tree_path;
+            GtkTreeSelection* tree_sel;
+            gtk_tree_view_get_path_at_pos( GTK_TREE_VIEW( view ),
+                                           evt->x, evt->y, &tree_path, NULL, NULL, NULL );
+            tree_sel = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
+
+            if ( tree_path && tree_sel && 
+                        !gtk_tree_selection_path_is_selected( tree_sel, tree_path ) )
+            {
+                gtk_tree_selection_unselect_all( tree_sel );
+                gtk_tree_selection_select_path( tree_sel, tree_path );
+            }
+            gtk_tree_path_free( tree_path );
+            
             GtkWidget* popup;
             GtkUIManager* menu_mgr;
             GtkActionGroup* action_group = gtk_action_group_new ("PopupActions");
@@ -905,12 +982,33 @@ static gboolean on_view_button_press( GtkTreeView* view, GdkEventButton* evt, Fi
     {
         if( evt->button == 1 )  /* left double click */
         {
+            on_open_files( NULL, data );
             return TRUE;
         }
     }
     return FALSE;
 }
 
+void on_use_size_lower_toggled( GtkWidget* widget, FindFile* data )
+{
+    gtk_widget_set_sensitive( data->size_lower,
+                    gtk_toggle_button_get_active( 
+                    GTK_TOGGLE_BUTTON( data->use_size_lower ) ) );
+    gtk_widget_set_sensitive( data->size_lower_unit, 
+                    gtk_toggle_button_get_active( 
+                    GTK_TOGGLE_BUTTON( data->use_size_lower ) ) );
+}
+
+void on_use_size_upper_toggled( GtkWidget* widget, FindFile* data )
+{
+    gtk_widget_set_sensitive( data->size_upper,
+                    gtk_toggle_button_get_active( 
+                    GTK_TOGGLE_BUTTON( data->use_size_upper ) ) );
+    gtk_widget_set_sensitive( data->size_upper_unit, 
+                    gtk_toggle_button_get_active( 
+                    GTK_TOGGLE_BUTTON( data->use_size_upper ) ) );
+}
+
 void fm_find_files( const char** search_dirs )
 {
     FindFile* data = g_slice_new0(FindFile);
@@ -918,11 +1016,25 @@ void fm_find_files( const char** search_dirs )
     GtkTreeViewColumn* col;
     GtkWidget *add_folder_btn, *remove_folder_btn, *img;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+    GtkBuilder* builder = _gtk_builder_new_from_file( PACKAGE_UI_DIR "/find-files2.ui", NULL );
+#else
     GtkBuilder* builder = _gtk_builder_new_from_file( PACKAGE_UI_DIR "/find-files.ui", NULL );
+#endif
     data->win = (GtkWidget*)gtk_builder_get_object( builder, "win" );
     g_object_set_data_full( G_OBJECT( data->win ), "find-files", data, (GDestroyNotify)free_data );
 
-    gtk_window_set_icon_name( GTK_WINDOW( data->win ), GTK_STOCK_FIND );
+    GdkPixbuf* icon = NULL;
+    GtkIconTheme* theme = gtk_icon_theme_get_default();
+    if ( theme )
+        icon = gtk_icon_theme_load_icon( theme, "spacefm-find", 48, 0, NULL );
+    if ( icon )
+    {
+        gtk_window_set_icon( GTK_WINDOW( data->win ), icon );
+        g_object_unref( icon );
+    }
+    else
+        gtk_window_set_icon_name( GTK_WINDOW( data->win ), GTK_STOCK_FIND );
 
     /* search criteria pane */
     data->search_criteria = (GtkWidget*)gtk_builder_get_object( builder, "search_criteria" );
@@ -947,6 +1059,12 @@ void fm_find_files( const char** search_dirs )
     data->size_upper = (GtkWidget*)gtk_builder_get_object( builder, "size_upper" );
     data->size_lower_unit = (GtkWidget*)gtk_builder_get_object( builder, "size_lower_unit" );
     data->size_upper_unit = (GtkWidget*)gtk_builder_get_object( builder, "size_upper_unit" );
+    g_signal_connect( data->use_size_lower, "toggled",
+                            G_CALLBACK( on_use_size_lower_toggled ), data );
+    g_signal_connect( data->use_size_upper, "toggled",
+                            G_CALLBACK( on_use_size_upper_toggled ), data );
+    on_use_size_lower_toggled( data->use_size_lower, data );
+    on_use_size_upper_toggled( data->use_size_upper, data );
 
     data->date_limit = (GtkWidget*)gtk_builder_get_object( builder, "date_limit" );
     data->date1 = (GtkWidget*)gtk_builder_get_object( builder, "date1" );
@@ -992,7 +1110,8 @@ void fm_find_files( const char** search_dirs )
     if( app_settings.single_click )
     {
         exo_tree_view_set_single_click( EXO_TREE_VIEW( data->result_view ), TRUE );
-        exo_tree_view_set_single_click_timeout( EXO_TREE_VIEW( data->result_view ), 400 );
+        exo_tree_view_set_single_click_timeout( EXO_TREE_VIEW( data->result_view ),
+                                                        SINGLE_CLICK_TIMEOUT );
     }
     gtk_widget_show( data->result_view );
     gtk_container_add( (GtkContainer*)gtk_builder_get_object(builder, "result_scroll"), data->result_view );
