@@ -255,7 +255,7 @@ static void on_combo_change( GtkComboBox* combo, gpointer user_data )
             VFSMimeType* mime = (VFSMimeType*)user_data;
             parent = gtk_widget_get_toplevel( GTK_WIDGET( combo ) );
             action = (char *) ptk_choose_app_for_mime_type( GTK_WINDOW(parent),
-                                                   mime, FALSE );
+                                                   mime, FALSE, TRUE, TRUE, TRUE );
             if( action )
             {
                 gboolean exist = FALSE;
@@ -458,9 +458,8 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
      * ,the selected file is a folder, or its type is unknown.
      */
     if( ! same_type ||
-          vfs_file_info_is_dir( file ) ||
           vfs_file_info_is_desktop_entry( file ) ||
-          vfs_file_info_is_unknown_type( file ) ||
+        /*  vfs_file_info_is_unknown_type( file ) || */
           vfs_file_info_is_executable( file, NULL ) )
     {
         /* if open with shouldn't show, destroy it. */
@@ -634,10 +633,17 @@ GtkWidget* file_properties_dlg_new( GtkWindow* parent,
             char* target_path = g_file_read_link( disp_path, NULL );
             if ( target_path )
             {
+                gtk_entry_set_text( GTK_ENTRY( target ), target_path );
+                if ( target_path[0] && target_path[0] != '/' )
+                {
+                    // relative link to absolute
+                    char* str = target_path;
+                    target_path = g_build_filename( dir_path, str, NULL );
+                    g_free( str );
+                }
                 if ( !g_file_test( target_path, G_FILE_TEST_EXISTS ) )
                     gtk_label_set_text( GTK_LABEL( mime_type ),
                                                     _("( broken link )") );
-                gtk_entry_set_text( GTK_ENTRY( target ), target_path );
                 g_free( target_path );
             }
             else
