@@ -1194,6 +1194,7 @@ static void cb_exec_child_watch( GPid pid, gint status, VFSFileTask* task )
     gboolean bad_status = FALSE;
     g_spawn_close_pid( pid );
     task->exec_pid = 0;
+    task->child_watch = 0;
 
     if ( status )
     {
@@ -1232,7 +1233,7 @@ static gboolean cb_exec_out_watch( GIOChannel *channel, GIOCondition cond,
 {
 
 /*
-printf("cb_exec_out_watch %d\n", channel);
+printf("cb_exec_out_watch %p\n", channel);
 if ( cond & G_IO_IN )
     printf("    G_IO_IN\n");
 if ( cond & G_IO_OUT )
@@ -1765,7 +1766,9 @@ static void vfs_file_task_exec( char* src_file, VFSFileTask* task )
         argv[a++] = g_strdup( use_su );
         if ( strcmp( task->exec_as_user, "root" ) )
         {
-            if ( strcmp( use_su, "/bin/su" ) )
+            if ( !strcmp( use_su, "/usr/bin/su-to-root" ) )
+                argv[a++] = g_strdup( "-p" );
+            else if ( strcmp( use_su, "/bin/su" ) )
                 argv[a++] = g_strdup( "-u" );
             argv[a++] = g_strdup( task->exec_as_user );
         }
@@ -1796,6 +1799,15 @@ static void vfs_file_task_exec( char* src_file, VFSFileTask* task )
         else if ( !strcmp( use_su, "/usr/bin/gnomesu" )
                                         || !strcmp( use_su, "/usr/bin/xdg-su" ) )
         {
+            // gnomesu
+            argv[a++] = g_strdup( "-c" );
+            single_arg = TRUE;
+        }
+        else if ( !strcmp( use_su, "/usr/bin/su-to-root" ) )
+        {
+            // su-to-root
+            if ( !terminal )
+                argv[a++] = g_strdup( "-X" );  // command is a X11 program
             argv[a++] = g_strdup( "-c" );
             single_arg = TRUE;
         }
@@ -1937,7 +1949,7 @@ static void vfs_file_task_exec( char* src_file, VFSFileTask* task )
     task->exec_pid = pid;
 
     // catch termination (always is run in the main loop thread)
-    guint child_watch = g_child_watch_add( pid,
+    task->child_watch = g_child_watch_add( pid,
                                     (GChildWatchFunc)cb_exec_child_watch, task );
 
     // create channels for output
@@ -2235,6 +2247,7 @@ VFSFileTask* vfs_task_new ( VFSFileTaskType type,
     task->exec_browser = NULL;
     task->exec_desktop = NULL;
     task->exec_pid = 0;
+    task->child_watch = 0;
     task->exec_is_error = FALSE;
     task->exec_scroll_lock = FALSE;
     task->exec_write_root = FALSE;
